// Server.cpp (Server.EXE)
// OCL Sample - Server Apllication for Balloons


#include "..\Source\server.hpp"

#ifndef __IBMCPP__
  #include <OList.c>
  #include <OThread.c>
#endif

// globals

static NBPAGE pages[] =
{
 {"Control Panel", "Control Panel", ID_CONTROL, CONTROL_TXT1, FALSE, BKA_MAJOR, 0, NULL },
 {"Count", "Count", ID_NUM, NUM_TXT1, FALSE, BKA_MAJOR, 0, NULL },
 {"Speed", "Speed", ID_NUM, NUM_TXT1, FALSE, BKA_MAJOR, 0, NULL },
};



BalloonServer server;

// code

void main(void)
{
 try
  {
   server.init();
  }

 catch(OPMException &err)
  {
   err.error(err.errRes);
   _exit(1);
  }

 server.run();
}



void BalloonServer::init()
{
 try
  {
   svrSem = new ONSem(BALL_SEM);
  }
 catch(...)
  {
   throw OPMException(SERVER_RUNNING, 0);
  }

 Buffer = &pipeMessage;
 if (!beginPiping(BALL_PIPE))
   throw OPMException(PIPE_FAILED, 0);

 pages[0].cmd = &firstPage;
 Pages.addLast(&pages[0]);

 pages[1].cmd = &secondPage;
 Pages.addLast(&pages[1]);

 pages[2].cmd = &thirdPage;
 Pages.addLast(&pages[2]);

 if (!createBook("Balloons Server", 260, 160, NB_STANDARD))
   throw OPMException(BOOK_FAILED, 0);

 showFrame();
}



// notebook pages

// control panel

BOOL BalloonServer::OCommand(ULONG msg, MPARAM mp1, MPARAM mp2)
{
 switch(msg)
  {
   case WM_CLOSE:
    if (connectedClients > 0) {
      pipeMessage.Type = PIPE_SERVER_EXIT;
      pipeMessage.Data = 0;
      postPipe(&pipeMessage, 0); }   // post to all clients
    OBook::OCommand(msg, mp1, mp2);
    stopPiping();
    forceQuit();
    break;

   default:
    return(OBook::OCommand(msg, mp1, mp2));
  }
 return(TRUE);
#ifdef __BCPLUSPLUS__
  #pragma warn -par
#endif
}
#ifdef __BCPLUSPLUS__
  #pragma warn .par
#endif

BOOL CtrlPanel::OCommand(ULONG msg, MPARAM mp1, MPARAM mp2)
{
 switch(msg)
  {
   case WM_COMMAND:
    switch(SHORT1FROMMP(mp1))
     {
      case CONTROL_STOP:
       if (server.connectedClients > 0) {
         server.pipeMessage.Type = PIPE_STOP_BALLOONS;
         server.pipeMessage.Data = 0;
         server.postPipe(&server.pipeMessage, 0); }   // post to all clients
       break;

      case CONTROL_START:
       if (server.connectedClients > 0) {
         server.pipeMessage.Type = PIPE_START_BALLOONS;
         server.pipeMessage.Data = 0;
         server.postPipe(&server.pipeMessage, 0); }   // post to all clients
       break;

      case CONTROL_XSERV:
       WinSendMsg(server.hwnd, WM_CLOSE, NULL, NULL);
       break;

      case CONTROL_XCLIENT:
       if (server.connectedClients > 0) {
         server.pipeMessage.Type = PIPE_STOP_CLIENT;
         server.pipeMessage.Data = 0;
         server.postPipe(&server.pipeMessage, 0); }   // post to all clients
       break;
      }
    break;

   default:
    return(FALSE);
  }
 return(TRUE);
#ifdef __BCPLUSPLUS__
  #pragma warn -par
#endif
}
#ifdef __BCPLUSPLUS__
  #pragma warn .par
#endif


BOOL NumSpeedControl::OCommand(ULONG msg, MPARAM mp1, MPARAM mp2)
{
 switch(msg)
  {
   case WM_INITDLG: {
    SWP swp;

    slider = new OSlider(NUM_SLIDER, hwnd, 5, 0, 5, 0,
                         SLS_HORIZONTAL | SLS_CENTER | SLS_SNAPTOINCREMENT |
                         SLS_BUTTONSLEFT | SLS_HOMELEFT | SLS_PRIMARYSCALE1 |
                         SLS_RIBBONSTRIP);
    WinQueryWindowPos(WinWindowFromID(hwnd, NUM_SLIDER_PLACE), &swp);
    slider->createSlider(swp.x, swp.y, swp.cx, swp.cy);
    slider->setTickSize(5);
    slider->setFont("8.Helv");
    slider->setSliderScaleText("1", 0);
    slider->setSliderScaleText("2", 1);
    slider->setSliderScaleText("3", 2);
    slider->setSliderScaleText("4", 3);
    slider->setSliderScaleText("5", 4);
    if (isSpeedCtrl)
      WinSetDlgItemText(hwnd, NUM_TXT1, "Select the speed of flying balloons...");
    break; }

   case WM_CONTROL:
    switch(SHORT2FROMMP(mp1)) {
     case SLN_CHANGE:
       if (server.connectedClients > 0) {
        if (isSpeedCtrl)
          server.pipeMessage.Type = PIPE_SET_SPEED;
        else
          server.pipeMessage.Type = PIPE_SET_NUM;
        server.pipeMessage.Data = 1 + slider->getSliderArmPos();
        server.postPipe(&server.pipeMessage, 0); }     // post to all clients
      break; }
    break;

   default:
    return(FALSE);
  }
 return(TRUE);
#ifdef __BCPLUSPLUS__
  #pragma warn -par
#endif
}
#ifdef __BCPLUSPLUS__
  #pragma warn .par
#endif



// end of source
