// OCL Sample
// Procstat.cpp

#ifdef __BCPLUSPLUS__
  #define __MSC
#endif


#include "..\Source\Procstat.hpp"

#ifndef __IBMCPP__
  #include <OList.c>
  #include <OSortedList.c>
  #include <OContainer.c>
#endif


void main(void)
{
 Procstat sample;

 try
  {
   sample.init(); 
  }

 catch(OPMException &err)
  {
   err.error("Application Exception", err.description.text);
   _exit(1);
  }
}




void Procstat::init()
{
 winText << "OS/2 Processes";
 if (!createFrame(winText.text))
   OThrowPM("Frame creation failed", 0, OException::unrecoverable);
 
 cnr = new PContainer(this);
 if (!cnr->createContainer())
   OThrowPM("Container creation failed", 0, OException::unrecoverable);
 cnr->setFont("8.Helv");

 showFrame();
 run();
}



BOOL Procstat::OCommand(ULONG msg, MPARAM mp1, MPARAM mp2)
{
 switch(msg)
  {
   case WM_SIZE:
     if (cnr)
       WinSetWindowPos(cnr->hwnd, 0, 0, 0, 
                       SHORT1FROMMP(mp2), SHORT2FROMMP(mp2), 
                       SWP_SIZE);
     break;

   case WM_COMMAND:
     switch(SHORT1FROMMP(mp1))
      {
       case PROCSTAT_QUIT:
        WinSendMsg(hwnd, WM_CLOSE, NULL, NULL);
        break; 

       case PROCSTAT_REFRESH:
        if (cnr) {
         cnr->freeContainer();
         cnr->setupItems(); }
        break;
      }
     break;

   case WM_CLOSE:
     forceQuit();
     break;

   default:
     return(OFrame::OCommand(msg, mp1, mp2));
  }
 return(TRUE);
#ifdef __BCPLUSPLUS__
  #pragma warn -par
#endif
}
#ifdef __BCPLUSPLUS__
  #pragma warn .par
#endif



BOOL PContainer::setupItems()
{
 PMINIRECORDCORE  prec;
 RECORDINSERT     ri;

 if (!getProcesses())         // OProcStat::getProcesses()
   OThrowPM("OProcStat::getProcesses() failed", 0, OException::unrecoverable);


 if ((prec = allocRecords(1)) == NULL)
   return(FALSE);

 prec->cb = sizeof(MINIRECORDCORE);
 prec->flRecordAttr = CRA_EXPANDED;
 prec->pszIcon = "OS2KRNL";
 prec->hptrIcon = hptr;

 memset(&ri, 0, sizeof(RECORDINSERT));
 ri.cb                = sizeof(RECORDINSERT);
 ri.pRecordOrder      = (PRECORDCORE) CMA_END;
 ri.pRecordParent     = (PRECORDCORE) NULL;
 ri.zOrder            = (USHORT) CMA_TOP;
 ri.cRecordsInsert    = 1;
 ri.fInvalidateRecord = FALSE;

 // insert them in one shot
 insertRecord(prec, &ri);

 return((insertChilds(prec, 0)) &&   // insert childs of kernel
        (insertChilds(prec, 1)) &&
        (setCnrInfo(CMA_LINESPACING | CMA_CXTREEINDENT)) &&
        (invalidateRecord()));
}


BOOL PContainer::insertChilds(PMINIRECORDCORE parentRecord, ULONG ulParent)
{
 PMINIRECORDCORE       prec;
 RECORDINSERT          ri;
 // we cannot use getFirst() because this member
 // is recursive and must not change the list cursors
 ListItem<OProcRecord> *cursor = (ListItem<OProcRecord>*)ProcList.first;

 while(cursor)
  {
   if (cursor->item->PPid == ulParent)
    {
    if ((prec = allocRecords(1)) == NULL)
      return(FALSE);

     prec->cb = sizeof(MINIRECORDCORE);
     prec->flRecordAttr = CRA_EXPANDED;
     prec->pszIcon = cursor->item->Module.text;
     prec->hptrIcon = hptr;

     memset(&ri, 0, sizeof(RECORDINSERT));
     ri.cb                = sizeof(RECORDINSERT);
     ri.pRecordOrder      = (PRECORDCORE) CMA_END;
     ri.pRecordParent     = (PRECORDCORE) parentRecord;
     ri.zOrder            = (USHORT) CMA_TOP;
     ri.cRecordsInsert    = 1;
     ri.fInvalidateRecord = FALSE;
    
     if ((!insertRecord(prec, &ri)) ||
         (!insertChilds(prec, cursor->item->Pid)))
       return(FALSE);
    }
   cursor = (ListItem<OProcRecord>*) cursor->next;
  }
 return(TRUE);
}

// end of source
