// OCL Sample Application
// Template.cpp
// Implementation


// includes

#include "..\Source\Template.hpp"

#ifndef __IBMCPP__
  #include <OList.c>
  #include <OSortedList.c>
  #include <OThread.c>
  #include <OContainer.c>
#endif


// prototypes

void main(int argc, char *argv[]);


// information struct for buttonbar

static ButtonInfo buttoninfo[] =
{
 {SAMPLE_FILL, SAMPLE_FILL, "Fill Container", NULL, NULLHANDLE, (HBITMAP)NULL, (HPOINTER)NULL },
 {SAMPLE_EXCEPT, SAMPLE_EXCEPT, "Throw Exception from within a command handler",
  NULL, NULLHANDLE, (HBITMAP)NULL, (HPOINTER)NULL },
 {SAMPLE_INFO, SAMPLE_INFO, "Application Information", NULL, NULLHANDLE, (HBITMAP)NULL, (HPOINTER)NULL },
 {SAMPLE_HELP, SAMPLE_HELP, "Application Help", NULL, NULLHANDLE, (HBITMAP)NULL, (HPOINTER)NULL },
 {SAMPLE_QUIT, SAMPLE_QUIT, "Quit", NULL, NULLHANDLE, (HBITMAP)NULL, (HPOINTER)NULL }
};

// source

void main(int argc, char *argv[])
{
 OCLApp sample(argc, argv);

 try
  {
   sample.createWindows();
   sample.run();
  }

 catch(OPMException &err)
  {
   err.error(err.description.text);
  }
}


// public methods of OCLApp


void OCLApp::createWindows()
{
 ULONG counter;
 ULONG screenDimension;

 if (!createFrame("OCL Sample Application"))
   OThrowPM("Frame creation failed.", 0, OException::unrecoverable);

 setWindowText("OCL Sample Application");

 if ((menu = WinLoadMenu(frame, NULLHANDLE, SAMPLE_MENU)) == NULLHANDLE)
   OThrowPM("Could not load menu.", 0, OException::unrecoverable);

// setup the buttonbar

 buttonbar = new OButtonBar(this, OButtonBar::iconlike);
 screenDimension = (ULONG) WinQuerySysValue(HWND_DESKTOP, SV_CYSCREEN);
 buttonbar->setButtonDimensions(screenDimension > 600 ? 50 : 40, screenDimension > 600 ? 50 : 40);
 buttonbar->setPicSize(screenDimension > 600 ? 40 : 32);

 for (counter = 0; counter < sizeof(buttoninfo)/sizeof(ButtonInfo); counter++)
   buttonbar->buttons.addLast(&buttoninfo[counter]); // add all buttons to the buttonlist;


// setup container & indicator

 cnr = new MyContainer(this);
 indicator = new OIndicator(this);

// setup the statusline

 statusline = new OStatusLine(this); // default is 8.Helv, DT_LEFT | DT_VCENTER
 statusline->style |= FCF_BORDER;
 
// setup windows

 if(!buttonbar->createButtonBar())
   OThrowPM("ButtonBar creation failed.", 0, OException::unrecoverable);
 if(!cnr->init())
   OThrowPM("Container initialization failed.", 0, OException::unrecoverable);
 if(!indicator->createIndicator())
   OThrowPM("Indicator creation failed.", 0, OException::unrecoverable);
 if(!statusline->createStatusLine())
   OThrowPM("StatusLine creation failed.", 0, OException::unrecoverable);

 cnr->menu = WinLoadMenu(cnr->hwnd, NULLHANDLE, CNR_MENU);
 cnr->setTitle("OCL Sample Container");
 setSizePosFrame(HWND_TOP, sizepos, SWP_SHOW | SWP_ACTIVATE);
 initHelp();
}


void OCLApp::initHelp()
{
 OString helpName(callName);

 helpName - '\\';
 helpName + "\\TEMPLATE.HLP";
 initializeHelp(helpName.text, "OCL Sample Help", frame);
}


// resize method

void OCLApp::resizeClients(MPARAM sizeparms)
{
 ULONG width  = SHORT1FROMMP(sizeparms);
 ULONG height = SHORT2FROMMP(sizeparms);
 ULONG iPos   = indicator->getSliderArmPos();

 if ((!buttonbar) || (!statusline))
   return;

 WinSetWindowPos(buttonbar->frame, HWND_TOP, 0, 
                 height - buttonbar->getHeight(),
                 width, 
                 buttonbar->getHeight(), 
                 SWP_SIZE | SWP_MOVE);
 WinSetWindowPos(cnr->hwnd, HWND_TOP, 0,
                 statusline->getHeight(),
                 width,
                 height - buttonbar->getHeight() - statusline->getHeight() - 50,
                 SWP_SIZE | SWP_MOVE);
 WinSetWindowPos(indicator->hwnd, HWND_TOP, 0,
                 height - buttonbar->getHeight() - 50,
                 width,
                 50,
                 SWP_SIZE | SWP_MOVE);
 indicator->setSliderArmPos(iPos);
 WinSetWindowPos(statusline->frame, HWND_TOP, 0, 0,
                 width,
                 statusline->getHeight(),
                 SWP_SIZE | SWP_MOVE);
}

// CommandHandler for our frame window


BOOL OCLApp::OCommand(ULONG msg, MPARAM mp1, MPARAM mp2)
{
 switch(msg)
  {
// messages from MyContainer
   case WM_CNR_FILLED:
    cnr->enable();
    buttonbar->enableButton(SAMPLE_FILL, TRUE);
    break;

   case WM_CNR_STATUS:
    statusline->setText((PSZ)PVOIDFROMMP(mp1));
    break;

   case WM_CNR_PERCENTAGE:
    indicator->setSliderArmPos(SHORT1FROMMP(mp1));
    break;

   case WM_CNR_FILLCNR:
    cnr->setupDetails();
    cnr->setupItems(); 
    break;

// frame messages
   case WM_SIZE:
    resizeClients(mp2);
    break;

   case WM_CONTROL:
    switch(SHORT1FROMMP(mp1))
     {
      case OBUTTONBAR_MOVEOVER:
       statusline->setText((PSZ) PVOIDFROMMP(mp2));
       wasOutOfContainer = TRUE;
       break;
     }
    break;

   case WM_BUTTON2DOWN:
     WinPopupMenu(frame, frame, cnr->menu, SHORT1FROMMP(mp1), SHORT2FROMMP(mp1), CNR_MENU,
                  PU_NONE | PU_KEYBOARD | PU_MOUSEBUTTON1 | PU_HCONSTRAIN | PU_VCONSTRAIN);
     break;

   case WM_HITTEST:
     if (wasOutOfContainer)
      {
       WinQueryWindowPos(cnr->hwnd, &sizepos);
       if ((SHORT2FROMMP(mp1) > statusline->getHeight()) &&
           (SHORT2FROMMP(mp1) < (statusline->getHeight() + sizepos.cy)))
        {
         statusline->setText("OCL Sample Container");
         wasOutOfContainer = FALSE;
        }
      }
     break;

   case WM_COMMAND:
    switch(SHORT1FROMMP(mp1))
     {
      case SAMPLE_EXCEPT:
       OThrowPM("This exception was thrown from within OCLApp::OCommand.", 
                0, OException::unrecoverable);

      case SAMPLE_FILL:
       WinPostMsg(hwnd, WM_COMMAND, MPFROMSHORT(CNR_REFILL), NULL);
       break;

      case SAMPLE_QUIT:
       postQuit();
       break;

      case SAMPLE_INFO:
       infoDialog = new InfoDlg(this);
       infoDialog->createDlg();
       break;

      case SAMPLE_HELP:
       helpRequest(PANEL_MAIN);
       break;

      case CNR_ICONVIEW:
       cnr->setView(CA_CONTAINERTITLE | CA_TITLESEPARATOR | CV_ICON);
       break;
      case CNR_TEXTVIEW:
       cnr->setView(CA_CONTAINERTITLE | CA_TITLESEPARATOR | CV_TEXT);
       break;
      case CNR_NAMEVIEW:
       cnr->setView(CA_CONTAINERTITLE | CA_TITLESEPARATOR | CV_NAME);
       break;
      case CNR_DETAILVIEW:
       cnr->setView(CA_CONTAINERTITLE | CA_TITLESEPARATOR | CV_DETAIL | CA_DETAILSVIEWTITLES);
       break;
      case CNR_MINIICONVIEW:
       cnr->setView(CA_CONTAINERTITLE | CA_TITLESEPARATOR | CV_ICON | CV_MINI);
       break;
      case CNR_MININAMEVIEW:
       cnr->setView(CA_CONTAINERTITLE | CA_TITLESEPARATOR | CV_NAME | CV_MINI);
       break;

      case CNR_REFILL:
       cnr->cleanup();
       cnr->fillThread.run();
       cnr->disable();
       buttonbar->enableButton(SAMPLE_FILL, FALSE);
       break;
     }
    break;

   case WM_HELP:
    helpRequest(PANEL_MAIN);
    break;

// draw message from OIndicator
   case WM_DRAWITEM: {
    POWNERITEM item = (POWNERITEM) mp2;
    switch(item->idItem)
     {
      case SDA_RIBBONSTRIP:
       indicator->drawSliderElement(item, CLR_RED);
       break;
      case SDA_SLIDERSHAFT:
       indicator->drawSliderElement(item, SYSCLR_WINDOW);
       break;
     }
    break; }

   default:
    return(FALSE);
  }
 return(TRUE);
}


// command handler for information dialog

BOOL InfoDlg::OCommand(ULONG msg, MPARAM mp1, MPARAM mp2)
{
 switch(msg)
  {
   case WM_INITDLG:
    centerDlg(CENTER_ONSCREEN);
    showDlg();
    break;

   case WM_COMMAND:
    WinPostMsg(hwnd, WM_CLOSE, NULL, NULL);
    break;

   case WM_CLOSE:
   case WM_DESTROY:
    hideDlg();
    app->infoDialog = NULL;
    delete this;
    break;

   default:
    return(FALSE);
  }
 return(TRUE);
#ifdef __BCPLUSPLUS__
  #pragma warn -par
#endif
}
#ifdef __BCPLUSPLUS__
  #pragma warn .par
#endif

// end of source
