// OCL Sample Application
// Template.hpp
// Class Definitions

#include"..\Source\Template.h"

#define OINCL_LIST
#define OINCL_SORTEDLIST
#define OINCL_OAPP
#define OINCL_OFRAME
#define OINCL_FCL
#define OINCL_OSYSINFO
#define OINCL_ODIALOG
#define OINCL_OMSGS
#define OINCL_OSTATLINE
#define OINCL_OBUTTONBAR
#define OINCL_OHELP
#define OINCL_OCONTAINER
#define OINCL_OSLIDER
#define OINCL_OINDICATOR
#define OINCL_OTHREAD
#define OINCL_OPMEXCEPTION


#include <ocl.hpp>

typedef class OCLApp *pOCLApp;
typedef class InfoDlg *pInfoDlg;


class InfoDlg : public ODialog
{
 private:
   pOCLApp  app;

 public:
   InfoDlg(pOCLApp application) 
      : app(application), ODialog(SAMPLE_DLG)
      {}
   virtual ~InfoDlg() {}

   BOOL OCommand(ULONG msg, MPARAM mp1, MPARAM mp2);
};


typedef struct CnrRecord *pCnrRecord;

struct CnrRecord
{
 RECORDCORE recordCore;
 PSZ        Name;
 ULONG      Position;
};

typedef struct MyRecord *pMyRecord;

struct MyRecord
{
 OString    Name;
 ULONG      Position;
};


typedef class slMyRecord *pslMyRecord;


class slMyRecord : public SortedList<MyRecord>
{
 public:
   slMyRecord() {}
   ~slMyRecord() {}

   BOOL isLess(const MyRecord *first, const MyRecord *second)
    {
     return(strcmpi(first->Name.text, second->Name.text) <= 0);
    }
};

typedef class MyContainer *pMyContainer;

class MyContainer 
   : public OContainer<CnrRecord, RECORDCORE>,
     public slMyRecord
{
 public:
   OThread<MyContainer> fillThread;
   HWND                 menu;

   MyContainer(pOFrame Parent)
     : OContainer<CnrRecord, RECORDCORE>(Parent, SAMPLE_CNR,
                  CCS_AUTOPOSITION,
                  CV_DETAIL | CA_DETAILSVIEWTITLES |
                  CA_CONTAINERTITLE | CA_TITLESEPARATOR),
       fillThread(this, &MyContainer::collectFiles, 32768) {}

   virtual ~MyContainer()
     {
      cleanup();
     }
 
   BOOL init();
   VOID cleanup();
   BOOL setupDetails();
   BOOL setupItems();
   void collectFiles();
};

typedef class OCLApp *pOCLApp;

class OCLApp : public OApp, public OHelp, public OFrame
{
 private:
   BOOL        wasOutOfContainer;  // pointer was out of container area?
   OButtonBar  *buttonbar;
   OStatusLine *statusline;
   OIndicator  *indicator;
   MyContainer *cnr;
   HWND        menu;

 public:
   pInfoDlg    infoDialog;

   OCLApp(const ULONG num, CHAR *Parms[]) 
     : OApp(num, Parms),
       OFrame(SAMPLE_MAIN,
              FCF_SIZEBORDER | FCF_ICON | FCF_TASKLIST | 
              FCF_SYSMENU | FCF_MINMAX | FCF_TITLEBAR | FCF_SHELLPOSITION,
              CS_SIZEREDRAW | CS_HITTEST),
       statusline(NULL),
       buttonbar(NULL),
       cnr(NULL),
       indicator(NULL),
       menu(NULLHANDLE),
       infoDialog(NULL),
       wasOutOfContainer(TRUE),
       OHelp(MAIN_HELP_TABLE, HELP_ERROR) {}

   virtual ~OCLApp() {
      if (infoDialog) delete infoDialog;
      if (buttonbar) delete buttonbar;
      if (cnr) delete cnr;
      if (indicator) delete indicator;
      if (statusline) delete statusline; }

   void createWindows();
   void initHelp();
   void resizeClients(MPARAM sizeparms);

// command handler

   BOOL OCommand(ULONG msg, MPARAM mp1, MPARAM mp2);
};


// end of source 
